/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jbi.descriptor.original;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URISyntaxException;
import java.net.URL;

import org.apache.commons.io.FileUtils;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.ow2.petals.jbi.descriptor.extension.JBIDescriptorExtensionBuilder;
import org.ow2.petals.jbi.descriptor.extension.exception.NoComponentNameDeployableServiceUnitException;
import org.ow2.petals.jbi.descriptor.extension.exception.NoIdentificationDeployableServiceUnitException;
import org.ow2.petals.jbi.descriptor.extension.exception.NotDeployableServiceUnitException;
import org.ow2.petals.jbi.descriptor.extension.exception.NotServiceUnitException;
import org.ow2.petals.jbi.descriptor.extension.exception.SeveralComponentNameDeployableServiceUnitException;
import org.ow2.petals.jbi.descriptor.extension.exception.SeveralIdentificationDeployableServiceUnitException;
import org.ow2.petals.jbi.descriptor.original.generated.Component;
import org.ow2.petals.jbi.descriptor.original.generated.Identification;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.zeroturnaround.zip.ZipUtil;

/**
 * @author Mathieu Carrolle - EBM WebSourcing
 */
public class JBIDescriptorBuilderTestCase {
	
	private final static String BCSOAP_JBI_DESC = "bcsoap.jbi.xml";

    private static final String EXPLODED_JBI_ARCHIVE = "exploded-component";

    private static final String COMPONENT_NAME = "identifier";

    @Rule
    public final TemporaryFolder tempFolder = new TemporaryFolder();

	/**
	 * Read a valid JBI descriptor, update it with characters with accent, next save it, and finally read the saved
	 * version.
	 */
    @Test
    public void buildXmlStringJBIDescriptor_001() throws JBIDescriptorException, IOException {
		
		// 1 - Read the original JBI descriptor
		final InputStream jbiDescriptorStream = Thread.currentThread().getContextClassLoader().getResourceAsStream(BCSOAP_JBI_DESC);
		final Jbi bcSoapJbiDesc =  JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(jbiDescriptorStream);
		
        try {
            JBIDescriptorExtensionBuilder.getInstance().getDeployableServiceUnitIndentification(bcSoapJbiDesc);
            fail(NotServiceUnitException.class.getName() + "not thrown");
        } catch (final NotServiceUnitException e) {
            // Expected exception
        }
        try {
            JBIDescriptorExtensionBuilder.getInstance().getDeployableServiceUnitTargetComponent(bcSoapJbiDesc);
            fail(NotServiceUnitException.class.getName() + "not thrown");
        } catch (final NotServiceUnitException e) {
            // Expected exception
        }

		// 2 - Update the read JBI descriptor with special character (with french accent)
		final Component comp = bcSoapJbiDesc.getComponent();
		assertNotNull(comp);
		final Identification ident = comp.getIdentification();
		assertNotNull(ident);
		ident.setDescription("My french characters: [éèçàù€ïîê]");
		
		// 3 - Temporary saved an instance of the read JBI descriptor
		final File tempJbiDescr = File.createTempFile("bcsoap", "jbi.xml");
		final OutputStream os = new FileOutputStream(tempJbiDescr);
		JBIDescriptorBuilder.getInstance().writeXMLJBIdescriptor(bcSoapJbiDesc, os);
		os.close();
		
		// 4 - Read the temporary JBI descriptor
		final InputStream is = new FileInputStream(tempJbiDescr);
		final Jbi tempJbiDesc = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(is);
		
		assertNotNull(tempJbiDesc);
		
	}

    /**
     * <p>
     * Try to pare the JBI descriptor of a deployable service unit, where the SU has no extensions
     * </p>
     * <p>
     * Expected result: an error occurs about missing extensions
     * </p>
     */
    @Test(expected = NotDeployableServiceUnitException.class)
    public void testDeployableSUJavaBuilding_NoExtensions() throws Exception {
        final URL xmlURL = this.getClass().getResource("/deployable-su-no-extensions.xml");
        assertNotNull("Deployable service unit JBI descriptor missing", xmlURL);
        final Jbi jbi = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
        JBIDescriptorExtensionBuilder.getInstance().getDeployableServiceUnitIndentification(jbi);
    }

    /**
     * <p>
     * Try to pare the JBI descriptor of a deployable service unit, where the SU has several identification sections
     * </p>
     * <p>
     * Expected result: an error occurs about identification sections
     * </p>
     */
    @Test(expected = SeveralIdentificationDeployableServiceUnitException.class)
    public void testDeployableSUJavaBuilding_SeveralIdentifications() throws Exception {
        final URL xmlURL = this.getClass().getResource("/deployable-su-several-identifications.xml");
        assertNotNull("Deployable service unit JBI descriptor missing", xmlURL);
        final Jbi jbi = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
        JBIDescriptorExtensionBuilder.getInstance().getDeployableServiceUnitIndentification(jbi);
    }

    /**
     * <p>
     * Try to pare the JBI descriptor of a deployable service unit, where the SU has several target component sections
     * </p>
     * <p>
     * Expected result: an error occurs about target component sections
     * </p>
     */
    @Test(expected = SeveralComponentNameDeployableServiceUnitException.class)
    public void testDeployableSUJavaBuilding_SeveralTargetComponents() throws Exception {
        final URL xmlURL = this.getClass().getResource("/deployable-su-several-target-components.xml");
        assertNotNull("Deployable service unit JBI descriptor missing", xmlURL);
        final Jbi jbi = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
        JBIDescriptorExtensionBuilder.getInstance().getDeployableServiceUnitTargetComponent(jbi);
    }

    /**
     * <p>
     * Try to pare the JBI descriptor of a deployable service unit, where the SU JBI descriptor contains unknown
     * extensions
     * </p>
     * <p>
     * Expected result: no error occurs
     * </p>
     */
    @Test
    public void testDeployableSUJavaBuilding_UnknownExtensions() throws Exception {
        final URL xmlURL = this.getClass().getResource("/deployable-su-unknown-extensions.xml");
        assertNotNull("Deployable service unit JBI descriptor missing", xmlURL);
        final Jbi jbi = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
        final org.ow2.petals.jbi.descriptor.extension.generated.Identification identification = JBIDescriptorExtensionBuilder.getInstance()
                .getDeployableServiceUnitIndentification(jbi);
        assertNotNull("Deployable service unit identification is missing", identification);
        assertEquals("su-name", identification.getName());
    }

    /**
     * <p>
     * Try to pare the JBI descriptor of a deployable service unit, where the SU JBI descriptor contains no
     * identification section
     * </p>
     * <p>
     * Expected result: an error occurs about missing identification section
     * </p>
     */
    @Test(expected = NoIdentificationDeployableServiceUnitException.class)
    public void testDeployableSUJavaBuilding_NoIdentification() throws Exception {
        final URL xmlURL = this.getClass().getResource("/deployable-su-no-identification.xml");
        assertNotNull("Deployable service unit JBI descriptor missing", xmlURL);
        final Jbi jbi = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
        JBIDescriptorExtensionBuilder.getInstance().getDeployableServiceUnitIndentification(jbi);
    }

    /**
     * <p>
     * Try to pare the JBI descriptor of a deployable service unit, where the SU JBI descriptor contains no target
     * component section
     * </p>
     * <p>
     * Expected result: an error occurs about missing target component section
     * </p>
     */
    @Test(expected = NoComponentNameDeployableServiceUnitException.class)
    public void testDeployableSUJavaBuilding_NoComponentName() throws Exception {
        final URL xmlURL = this.getClass().getResource("/deployable-su-no-target-component.xml");
        assertNotNull("Deployable service unit JBI descriptor missing", xmlURL);
        final Jbi jbi = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
        JBIDescriptorExtensionBuilder.getInstance().getDeployableServiceUnitTargetComponent(jbi);
    }

    /**
     * <p>
     * Try to parse and update the JBI descriptor of a standard service unit:
     * </p>
     * <p>
     * Expected result:
     * <ul>
     * <li>No error occurs parsing the JBI descriptor,</li>
     * <li>An error occurs getting the extensions associated to a deployable service unit,</li>
     * <li>No error occurs updating the JBI descriptor with extensions associated to a deployable service unit,</li>
     * <li>No error occurs saving the updated JBI descriptor,</li>
     * <li>No error occurs parsing the JBI descriptor sevad,</li>
     * <li>No error occurs getting the extensions associated to a deployable service unit.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testSUJavaBuilding() throws Exception {
        final URL xmlURL = this.getClass().getResource("/standard-su.xml");
        assertNotNull("Deployable service unit JBI descriptor missing", xmlURL);
        final Jbi jbi = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
        assertNotNull("It's not a SU JBI descriptor", jbi.getServices());

        try {
            JBIDescriptorExtensionBuilder.getInstance().getDeployableServiceUnitIndentification(jbi);
            fail(NotDeployableServiceUnitException.class.getName() + "not thrown");
        } catch (final NotDeployableServiceUnitException e) {
            // Expected exception
        }
        try {
            JBIDescriptorExtensionBuilder.getInstance().getDeployableServiceUnitTargetComponent(jbi);
            fail(NotDeployableServiceUnitException.class.getName() + "not thrown");
        } catch (final NotDeployableServiceUnitException e) {
            // Expected exception
        }

        // Update the JBI decriptor with extensions associated to a deployable service unit
        final String expectedDescription = "My expected description";
        final String expectedSuName = "my-expected-name";
        final String expectedTargetComponentName = "expected-component-name";
        final org.ow2.petals.jbi.descriptor.extension.generated.Identification suIdent = new org.ow2.petals.jbi.descriptor.extension.generated.Identification();
        suIdent.setName(expectedSuName);
        suIdent.setDescription(expectedDescription);
        JBIDescriptorExtensionBuilder.getInstance().setDeployableServiceUnitIdentification(jbi, suIdent);
        JBIDescriptorExtensionBuilder.getInstance().setDeployableServiceUnitTargetComponent(jbi, expectedTargetComponentName);

        // Save the JBI descriptor updated
        final File jbiFile = this.tempFolder.newFile();
        final OutputStream osJbi = new FileOutputStream(jbiFile);
        try {
            JBIDescriptorBuilder.getInstance().writeXMLJBIdescriptor(jbi, osJbi);
        } finally {
            osJbi.close();
        }

        // Parse the JBI descriptor saved
        final InputStream isJbi = new FileInputStream(jbiFile);
        try {
            final Jbi jbiSaved = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(isJbi);
            assertNotNull("It's not a SU JBI descriptor", jbi.getServices());
            final org.ow2.petals.jbi.descriptor.extension.generated.Identification savedSuIdent = JBIDescriptorExtensionBuilder.getInstance()
                    .getDeployableServiceUnitIndentification(jbiSaved);
            assertNotNull(savedSuIdent);
            assertEquals(expectedSuName, savedSuIdent.getName());
            assertEquals(expectedDescription, savedSuIdent.getDescription());
            assertEquals(expectedTargetComponentName,
                    JBIDescriptorExtensionBuilder.getInstance().getDeployableServiceUnitTargetComponent(jbiSaved));

        } finally {
            isJbi.close();
        }
    }

    /**
     * <p>
     * Try to parse and update the JBI descriptor of a deployable standard service unit:
     * </p>
     * <p>
     * Expected result:
     * <ul>
     * <li>No error occurs parsing the JBI descriptor,</li>
     * <li>No error occurs getting the extensions associated to a deployable service unit,</li>
     * <li>No error occurs updating the JBI descriptor with extensions associated to a deployable service unit,</li>
     * <li>No error occurs saving the updated JBI descriptor,</li>
     * <li>No error occurs parsing the JBI descriptor sevad,</li>
     * <li>No error occurs getting the extensions associated to a deployable service unit.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testDeployableSUJavaBuilding() throws Exception {
        final URL xmlURL = this.getClass().getResource("/deployable-su.xml");
        assertNotNull("Deployable service unit JBI descriptor missing", xmlURL);
        final Jbi jbi = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
        final org.ow2.petals.jbi.descriptor.extension.generated.Identification identification = JBIDescriptorExtensionBuilder.getInstance()
                .getDeployableServiceUnitIndentification(jbi);
        assertNotNull("Deployable service unit identification is missing", identification);
        assertEquals("su-name", identification.getName());
        final String targetComponent = JBIDescriptorExtensionBuilder.getInstance().getDeployableServiceUnitTargetComponent(jbi);
        assertNotNull("Deployable service unit target component is missing", targetComponent);
        assertEquals("petals-bc-soap", targetComponent);

        // Update the JBI decriptor with extensions associated to a deployable service unit
        final String expectedDescription = "My expected description";
        final String expectedSuName = "my-expected-name";
        final String expectedTargetComponentName = "expected-component-name";
        final org.ow2.petals.jbi.descriptor.extension.generated.Identification suIdent = new org.ow2.petals.jbi.descriptor.extension.generated.Identification();
        suIdent.setName(expectedSuName);
        suIdent.setDescription(expectedDescription);
        JBIDescriptorExtensionBuilder.getInstance().setDeployableServiceUnitIdentification(jbi, suIdent);
        JBIDescriptorExtensionBuilder.getInstance().setDeployableServiceUnitTargetComponent(jbi, expectedTargetComponentName);

        // Save the JBI descriptor updated
        final File jbiFile = this.tempFolder.newFile();
        final OutputStream osJbi = new FileOutputStream(jbiFile);
        try {
            JBIDescriptorBuilder.getInstance().writeXMLJBIdescriptor(jbi, osJbi);
        } finally {
            osJbi.close();
        }

        // Parse the JBI descriptor saved
        final InputStream isJbi = new FileInputStream(jbiFile);
        try {
            final Jbi jbiSaved = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(isJbi);
            assertNotNull("It's not a SU JBI descriptor", jbiSaved.getServices());
            final org.ow2.petals.jbi.descriptor.extension.generated.Identification savedSuIdent = JBIDescriptorExtensionBuilder.getInstance()
                    .getDeployableServiceUnitIndentification(jbiSaved);
            assertNotNull(savedSuIdent);
            assertEquals(expectedSuName, savedSuIdent.getName());
            assertEquals(expectedDescription, savedSuIdent.getDescription());
            assertEquals(expectedTargetComponentName,
                    JBIDescriptorExtensionBuilder.getInstance().getDeployableServiceUnitTargetComponent(jbiSaved));

        } finally {
            isJbi.close();
        }
    }

    /**
     * <p>
     * Load, from a file, the JBI descriptor of a JBI archive where:
     * <ul>
     * <li>the JBI archive is valid,</li>
     * <li>the JBI archive is located in a valid directory.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs.
     * </p>
     * 
     */
    @Test
    public void loadDescriptorFromFile() throws JBIDescriptorException, URISyntaxException {

        final URL packageURL = Thread.currentThread().getContextClassLoader().getResource(EXPLODED_JBI_ARCHIVE);
        assertNotNull("Exploded JBI archive is missing", packageURL);

        final Jbi jbi = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptorFromFolder(
                new File(packageURL.toURI()));

        assertJbi(jbi);

    }

    /**
     * <p>
     * Load, from a file, the JBI descriptor of a JBI archive where:
     * <ul>
     * <li>the JBI archive is valid,</li>
     * <li>the JBI archive is located in a valid directory containing a space.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs.
     * </p>
     */
    @Test
    public void loadDescriptorFromFile_DirNameContainsSapce() throws JBIDescriptorException, URISyntaxException,
            IOException {

        final File targetPackageDir = this.tempFolder.newFolder("dir with space");

        final URL initialPackageURL = Thread.currentThread().getContextClassLoader().getResource(EXPLODED_JBI_ARCHIVE);
        assertNotNull("Exploded JBI archive is missing", initialPackageURL);

        FileUtils.copyDirectory(new File(initialPackageURL.toURI()), targetPackageDir);

        final Jbi jbi = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptorFromFolder(targetPackageDir);

        assertJbi(jbi);
    }

    /**
     * <p>
     * Load, from an URL, the JBI descriptor of a JBI archive where:
     * <ul>
     * <li>the JBI archive is valid,</li>
     * <li>the JBI archive is located into an URL pointing to a valid directory.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs.
     * </p>
     */
    @Test
    public void loadDescriptorFromURL() throws JBIDescriptorException, IOException, URISyntaxException {

        final URL explodedpackageURL = Thread.currentThread().getContextClassLoader().getResource(EXPLODED_JBI_ARCHIVE);
        assertNotNull("Exploded JBI archive is missing", explodedpackageURL);

        final File packageFile = this.tempFolder.newFile("component.zip");
        ZipUtil.pack(new File(explodedpackageURL.toURI()), packageFile);

        final Jbi jbi = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptorFromArchive(packageFile);

        assertJbi(jbi);

    }

    /**
     * <p>
     * Load, from an URL, the JBI descriptor of a JBI archive where:
     * <ul>
     * <li>the JBI archive is valid,</li>
     * <li>the JBI archive is located into an URL pointing to a valid directory containing a space.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs.
     * </p>
     */
    @Test
    public void loadDescriptorFromURL_DirNameContainsSapce() throws JBIDescriptorException,
            URISyntaxException, IOException {

        final File targetPackageDir = this.tempFolder.newFolder("dir with space");

        final URL explodedpackageURL = Thread.currentThread().getContextClassLoader().getResource(EXPLODED_JBI_ARCHIVE);
        assertNotNull("Exploded JBI archive is missing", explodedpackageURL);

        final File packageFile = this.tempFolder.newFile("component.zip");
        ZipUtil.pack(new File(explodedpackageURL.toURI()), packageFile);

        FileUtils.copyFileToDirectory(packageFile, targetPackageDir);

        final File targetPackageFile = new File(targetPackageDir, packageFile.getName());
        final Jbi jbi = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptorFromArchive(targetPackageFile);

        assertJbi(jbi);
    }

    private final void assertJbi(final Jbi jbi) {

        assertNotNull(jbi);
        assertNotNull(jbi.getComponent());
        assertNotNull(jbi.getComponent().getIdentification());
        assertEquals(COMPONENT_NAME, jbi.getComponent().getIdentification().getName());
    }
}
